import datetime
import random

from vkbotfather.fatherbot.models.settings import SettingsBot


class Model(SettingsBot):

    def startListen(self):
        """Listens to messages and sends their responses to analyzes"""
        try:
            print('| Server is running\n| {0}'.format(datetime.datetime.now()))
            for event in self.vk_api.listenServer():
                if self.vk_api.isNewMessage(event):
                    self.giveResponse(event['object']['message']['text'], event['object']['message']['peer_id'])

        except KeyboardInterrupt:
            print('| Server shutdown\n| {0}'.format(datetime.datetime.now()))

    def giveResponse(self, event_text, event_user_id):
        """Takes a response and analyzes it. 
        This function processes requests from only one user. 
        You can change the response processing parameters."""

        print('| {2} :Bot got a message "{0}" <- @{1}'.format(event_text, event_user_id, datetime.datetime.now())) 
        
        response = self.messageAnalysis(event_text)
        if '.png' in response:
            self.sendImage(response, event_user_id)
        else:
            self.sendMessage(response, event_user_id)

    def sendMessage(self, msg, user_id):
        """Sends a text message to the user."""
        self.vk_api.sendMessage(
            peer_id=user_id,
            message=msg,
            random_id=random.randint(1, 10 ** 8)
        )

        print('| {2} :Bot send a message "{0}" -> @{1}'.format(msg, user_id, datetime.datetime.now()))

    def sendImage(self, image, user_id): #Work only one sendImage
        """Sends a image to the user."""
        self.vk_api.sendImage(
            peer_id=user_id, 
            path = image,
            random_id = random.randint(1, 10 ** 8)
        )

        print('| {2} :Bot send a image "{0}" -> @{1}'.format(image, user_id, datetime.datetime.now()))
    
    def messageAnalysis(self, message):
        """Analyzes messages and return response.
        
        If your response has line breaks, then use double quotes for this."""

        response = None
        for plugin in self.plugins:
            response = plugin(message)
            if response != None:
                return response

    def addPlugin(self, plugin):
        """Add only one plugin in plugins list."""

        self.plugins.append(plugin)
    
    def addPlugins(self, plugins):
        """Add plugins in plugins list."""

        for plugin in plugins:
            self.addPlugin(plugin)


import requests

class VKAPI:
    def __init__(self, token, group_id):
        self.token = token
        self.group_id = group_id

    def updateLongpoll(self):
        api_url = 'https://api.vk.com/method/groups.getLongPollServer?access_token={0}&group_id={1}&v=5.102'.format(self.token, self.group_id)
        response = requests.get(api_url).json()

        server_url = response['response']['server']
        key = response['response']['key']
        ts = response['response']['ts']

        return server_url, key, ts

    def checkLongpoll(self):
        server_url, key, ts = self.updateLongpoll()
        longoll_url = '{0}?act=a_check&key={1}&wait=25&mode=2&ts={2}'.format(server_url, key, ts)
        longoll_get = requests.get(longoll_url).json()
        ts = longoll_get['ts']
        updates = longoll_get['updates']

        return updates

    def listenServer(self):
        while True:
            for event in self.checkLongpoll():
                yield event

    def openFile(self, path):
        file = open(path, 'rb')
        
        return file


    def uploadImage(self, peer_id, path):
        server_url = 'https://api.vk.com/method/photos.getMessagesUploadServer?peer_id={0}&access_token={1}&v=5.103'.format(peer_id, self.token)
        upload_server = requests.get(server_url).json()

        image = self.openFile(path)

        post_url = upload_server['response']['upload_url']
        post_response = requests.post(post_url, files=dict(file=image)).json()
        server, photo, vk_hash = post_response['server'], post_response['photo'], post_response['hash']

        save_url = 'http://api.vk.com/method/photos.saveMessagesPhoto?photo={0}&server={1}&hash={2}&access_token={3}&v=5.103'.format(photo, server, vk_hash, self.token)
        save_response  = requests.get(save_url).json()
        upload_image = 'photo{0}_{1}'.format(save_response['response'][0]['owner_id'], save_response['response'][0]['id'])

        return upload_image
    
    def sendImage(self, peer_id, path, random_id):
        photo = self.uploadImage(peer_id, path)
        url_send = 'https://api.vk.com/method/messages.send?attachment={0}&peer_id={1}&access_token={2}&random_id={3}&v=5.103'.format(photo, peer_id, self.token, random_id)
        response_send = requests.get(url_send).json()
        print(response_send)

    
    def sendMessage(self, message, peer_id, random_id):
        url_send = 'https://api.vk.com/method/messages.send?message={0}&peer_id={1}&access_token={2}&random_id={3}&v=5.103'.format(message, peer_id, self.token, random_id)
        response_send = requests.get(url_send).json()

    def isNewMessage(self, event):
        if event['type'] == 'message_new':
            return True
        else:
            return False